@@ -334,7 +334,7 @@ class OverviewButton(GObject.Object):
         if remote_machine.status == RemoteStatus.INIT_CONNECTING:
             self.overview_user_connecting_spinner.show()
             self.overview_user_status_icon.hide()
-            self.ip_label.set_text(str(self.remote_machine.ips))
+            self.ip_label.set_text(str(self.remote_machine.ip_info.ip4_address))
             if have_info:
                 self.button.set_tooltip_text(_("Connecting"))
             else:
@@ -371,7 +371,7 @@ class OverviewButton(GObject.Object):
         else:
             self.overview_user_hostname.set_text(self.remote_machine.display_hostname)
 
-        self.ip_label.set_text(str(self.remote_machine.ips))
+        self.ip_label.set_text(str(self.remote_machine.ip_info.ip4_address))
 
         if self.remote_machine.avatar_surface:
             self.avatar_image.set_from_surface(self.remote_machine.avatar_surface)
@@ -601,7 +601,7 @@ class WarpWindow(GObject.Object):
         for button in self.user_list_box.get_children():
             joined = " ".join([button.remote_machine.display_name,
                                ("%s@%s" % (button.remote_machine.user_name, button.remote_machine.hostname)),
-                               button.remote_machine.ips])
+                               button.remote_machine.ip_info.ip4_address])
             normalized_contents = GLib.utf8_normalize(joined, len(joined), GLib.NormalizeMode.DEFAULT).lower()
 
             if normalized_query in normalized_contents:
@@ -699,10 +699,10 @@ class WarpWindow(GObject.Object):
         Gtk.drag_finish(context, True, False, _time)
         self.drop_pending = False
 
-    def update_local_user_info(self, ips=util.IPAddresses("0.0.0.0", None), iface=""):
+    def update_local_user_info(self, ip="0.0.0.0", iface=""):
         self.app_local_name_label.set_text(util.get_local_name())
         self.app_iface_label.set_text(iface)
-        self.app_ip_label.set_text(str(ips))
+        self.app_ip_label.set_text(ip)
 
     def menu_quit(self, widget, data=None):
         self.display_shutdown()
@@ -846,7 +846,7 @@ class WarpWindow(GObject.Object):
         else:
             self.user_hostname_label.set_text(remote.display_hostname)
 
-        self.user_ip_label.set_text(str(remote.ips))
+        self.user_ip_label.set_text(str(remote.ip_info.ip4_address))
 
         if remote.avatar_surface != None:
             self.user_avatar_image.set_from_surface(remote.avatar_surface)
@@ -1008,8 +1008,7 @@ class WarpApplication(Gtk.Application):
 
         self.current_port = None
         self.current_auth_port = None
-        self.current_ips = None
-        self.current_iface = None
+        self.current_ip_info = None
 
     def do_startup(self):
         Gtk.Application.do_startup(self)
@@ -1057,30 +1056,14 @@ class WarpApplication(Gtk.Application):
 
     def start_network_monitor(self):
         self.netmon = networkmonitor.get_network_monitor()
-
-        if not self.netmon.ready():
-            self.netmon.connect("ready", self.netmon_ready)
-        else:
-            self.netmon_ready(self.netmon)
-
-    def netmon_ready(self, netmon):
-        logging.debug("Network client initialized")
-
-        try:
-            self.netmon.disconnect_by_func(self.netmon_ready)
-        except:
-            pass
-
+        self.netmon.start()
         self.netmon.connect("state-changed", self.network_state_changed)
-        self.netmon.connect("details-changed", self.network_details_changed)
+
         self.new_server()
 
     def network_state_changed(self, netmon, online):
         self.new_server()
 
-    def network_details_changed(self, netmon):
-        self.new_server()
-
     def new_server(self):
         if self.server_starting:
             logging.debug("Trying to start server while server already starting, will check later")
@@ -1094,12 +1077,11 @@ class WarpApplication(Gtk.Application):
 
         self.current_port = prefs.get_port()
         self.current_auth_port = prefs.get_auth_port()
-        self.current_ips = self.netmon.get_ips()
-        self.current_iface = self.netmon.get_current_iface() or ""
+        self.current_ip_info = self.netmon.get_current_ip_info()
 
-        logging.debug("New server requested for '%s' (%s)", self.current_iface, self.current_ips)
+        logging.debug("New server requested for '%s' (%s)", self.current_ip_info.iface, self.current_ip_info.ip4_address)
 
-        self.window.update_local_user_info(self.current_ips, self.current_iface)
+        self.window.update_local_user_info(self.current_ip_info.ip4_address, self.current_ip_info.iface)
 
         self.window.clear_remotes()
 
@@ -1117,7 +1099,7 @@ class WarpApplication(Gtk.Application):
             except:
                 pass
 
-            auth_singleton.update(self.current_ips, self.current_port)
+            auth_singleton.update(self.current_ip_info, self.current_port)
             auth_singleton.connect("group-code-changed", self.on_group_code_changed)
 
             if not self.netmon.online:
@@ -1126,7 +1108,7 @@ class WarpApplication(Gtk.Application):
                 self.window.show_no_network()
                 return
 
-            self.server = server.Server(self.current_iface, self.current_ips, self.current_port, self.current_auth_port)
+            self.server = server.Server(self.current_ip_info, self.current_port, self.current_auth_port)
             self.server.connect("server-started", self._server_started)
             self.server.connect("remote-machine-added", self._remote_added)
             self.server.connect("remote-machine-removed", self._remote_removed)
